@@ -265,7 +265,7 @@ if (inherits(yfdPar, "fdPar") || inherits(yfdPar, "fd")) {
             mj2    <- mj2 + ncoefj
             indexj <- mj1:mj2
             coefj  <- betacoef[indexj]
-            betafdj$coefs <- coefj
+            betafdj$coefs <- as.matrix(coefj)
             betafdParj$fd <- betafdj
         }
         betaestlist[[j]] <- betafdParj
@@ -406,7 +406,7 @@ if (inherits(yfdPar, "fdPar") || inherits(yfdPar, "fd")) {
         betacoefj <- betacoef[indexj]
         if (inherits(xfdj, "fd")) {
             betaestfdj       <- betafdj
-            betaestfdj$coefs <- betacoefj
+            betaestfdj$coefs <- as.matrix(betacoefj)
             betaestfdParj    <- betafdParj
             betaestfdParj$fd <- betaestfdj
             betaestlist[[j]] <- betaestfdParj
